/*
 * Purpose: Serializable wxGridTable, based on wxStringGridTable
 * Author:  Frank Bu
 * Created: 2002
 */

#ifndef ADDRESS_GRID_TABLE_H
#define ADDRESS_GRID_TABLE_H

#include <wx/wx.h>
#include <wx/grid.h>
#include <wx/datstrm.h>

#include "AddressManager.h"
#include "ColumnField.h"

WX_DECLARE_OBJARRAY(wxString, StringArray);
WX_DECLARE_OBJARRAY(StringArray, StringGrid);

/**
 * Base class with some helper methods for AddressGridTable.
 */
class wxGridTableBaseWithNotifyMethods : public wxGridTableBase
{
public:
	/**
	 * Sends a wxGRIDTABLE_REQUEST_VIEW_GET_VALUES wxGridTableMessage
	 * to the view, if not NULL.
	 */
	void RequestViewGetValues();

	/**
	 * Sends a wxGRIDTABLE_REQUEST_VIEW_SEND_VALUES wxGridTableMessage
	 * to the view, if not NULL.
	 */
	void RequestViewSendValues();

	/**
	 * Sends a wxGRIDTABLE_NOTIFY_ROWS_INSERTED wxGridTableMessage
	 * to the view, if not NULL.
	 * \param pos Start position, from which the rows were inserted.
	 * \param numRows Number of rows, which were inserted.
	 */
	void NotifyRowsInserted(int pos, int numRows);

	/**
	 * Sends a wxGRIDTABLE_NOTIFY_ROWS_APPENDED wxGridTableMessage
	 * to the view, if not NULL.
	 * \param numRows Number of rows, which were appended to the end.
	 */
	void NotifyRowsAppended(int numRows);

	/**
	 * Sends a wxGRIDTABLE_NOTIFY_ROWS_DELETED wxGridTableMessage
	 * to the view, if not NULL.
	 * \param pos Start position, from which the rows were deleted.
	 * \param numRows Number of rows, which were deleted.
	 */
	void NotifyRowsDeleted(int pos, int numRows);

	/**
	 * Sends a wxGRIDTABLE_NOTIFY_COLS_INSERTED wxGridTableMessage
	 * to the view, if not NULL.
	 * \param pos Start position, from which the columns were inserted.
	 * \param numCols Number of columns, which were inserted.
	 */
	void NotifyColsInserted(int pos, int numCols);

	/**
	 * Sends a wxGRIDTABLE_NOTIFY_COLS_APPENDED wxGridTableMessage
	 * to the view, if not NULL.
	 * \param numCols Number of columns, which were appended to the end.
	 */
	void NotifyColsAppended(int numCols);

	/**
	 * Sends a wxGRIDTABLE_NOTIFY_COLS_DELETED wxGridTableMessage
	 * to the view, if not NULL.
	 * \param pos Start position, from which the columns were deleted.
	 * \param numCols Number of columns, which were deleted.
	 */
	void NotifyColsDeleted(int pos, int numCols);
};

/**
 * Serializable wxGridTable.
 */
class AddressGridTable : public wxGridTableBaseWithNotifyMethods
{
public:
    // these are pure virtual in wxGridTableBase
    //
    int GetNumberRows();
    int GetNumberCols();
    wxString GetValue( int row, int col );
    void SetValue( int row, int col, const wxString& s );
    bool IsEmptyCell( int row, int col );

    // overridden functions from wxGridTableBase
    //
    void Clear();
    bool InsertRows( size_t pos = 0, size_t numRows = 1 );
    bool AppendRows( size_t numRows = 1 );
    bool DeleteRows( size_t pos = 0, size_t numRows = 1 );
    bool InsertCols( size_t pos = 0, size_t numCols = 1 );
    bool AppendCols( size_t numCols = 1 );
    bool DeleteCols( size_t pos = 0, size_t numCols = 1 );

    void SetColLabelValue( int col, const wxString& );
    wxString GetColLabelValue( int col );

	/**
	 * Deletes and initializes the table with default values.
	 */
	void Initialize();

	/**
	 * Initializes the table from a stream.
	 * \param in The stream from which the table data will be loaded.
	 */
	void Initialize(wxDataInputStream& in);

	/**
	 * Saves the table to a stream.
	 * \param in The stream to which the table data will be saved.
	 */
	void serialize(wxDataOutputStream& out);

	/**
	 * Sort the table.
	 * \param col1 Index of the first column, which will be used for sorting.
	 * \param col2 Index of the second column, which will be used for sorting or -1, if only sorting with one column.
	 * \param col3 Index of the third column, which will be used for sorting or -1, if only sorting with two columns.
	 */
	void Sort(int col1, int col2, int col3);

	/**
	 * Rename, move, add and delete columns.
	 * \param column The new columns, referencing old columns, if moved or renamed.
	 */
	void ChangeColumns(ColumnFieldArray& column);

private:

	/**
	 * Clears all rows and columns.
	 */
	void ClearAll();

	/**
	 * The content of the table.
	 */
    StringGrid m_data;

	/**
	 * The column labels.
	 */
    StringArray m_colLabels;
};

#endif
